/*
 * Taken from Linux Device Drivers, 3rd Edition.
 * Corbet, Rubini, and Kroah-Hartman.
 */

#include <linux/init.h>
#include <linux/module.h>
#include <linux/sched.h>
#include <linux/kthread.h>
#include <linux/err.h>
MODULE_LICENSE("Dual BSD/GPL");

int int_parm = 0;
module_param(int_parm, int, 0);
MODULE_PARM_DESC(int_parm, "a silly string here");

static struct task_struct *khello_task;

static int hello_thread(void *arg)
{
	printk(KERN_ALERT "hello_thread %ld\n", (long)arg);
	do {
		set_current_state(TASK_UNINTERRUPTIBLE);
		schedule_timeout(10 * HZ);
		printk(KERN_ALERT "hello_thread %ld awakening\n", (long)arg);
	} while (!kthread_should_stop());
	printk(KERN_ALERT "hello_thread %ld exiting\n", (long)arg);
	return 0;
}

static int hello_init(void)
{
	int err;

	printk(KERN_ALERT "Hello, world!  int_parm = %d\n", int_parm);
	khello_task = kthread_run(hello_thread, (void *)(long)int_parm,
				  "hello_thread");
	if (IS_ERR(khello_task)) {
		err = PTR_ERR(khello_task);
		khello_task = NULL;
		return err;
	}
	return 0;
}

static void hello_exit(void)
{
	if (khello_task != NULL) {
		kthread_stop(khello_task);
	}
	khello_task = NULL;
	printk(KERN_ALERT "Goodbye, cruel world\n");
}

module_init(hello_init);
module_exit(hello_exit);
