/*
 * Taken from Linux Device Drivers, 3rd Edition.
 * Corbet, Rubini, and Kroah-Hartman.
 *
 * Prints out rcu_ctrlblk.completed -- useful to check if RCU grace
 * periods have stalled.
 */

#include <linux/init.h>
#include <linux/module.h>
#include <linux/sched.h>
#include <linux/kthread.h>
#include <linux/err.h>
#include <linux/rcupdate.h>
MODULE_LICENSE("Dual BSD/GPL");

int interval = 10;
module_param(interval, int, 0);
MODULE_PARM_DESC(interval, "interval between print (seconds)");

static struct task_struct *khello_task;

static int rcucompleted_thread(void *arg)
{
	do {
		set_current_state(TASK_UNINTERRUPTIBLE);
		schedule_timeout(interval * HZ);
		printk(KERN_ALERT "rcucompleted: rcu_ctrlblk.completed=%ld\n",
		       rcu_batches_completed());
	} while (!kthread_should_stop());
	printk(KERN_ALERT "rcucompleted %ld exiting\n", (long)arg);
	return 0;
}

static int hello_init(void)
{
	int err;

	printk(KERN_ALERT "rcucompleted: interval = %d seconds\n", interval);
	khello_task = kthread_run(rcucompleted_thread,
				  (void *)(long)interval,
				  "hello_thread");
	if (IS_ERR(khello_task)) {
		err = PTR_ERR(khello_task);
		khello_task = NULL;
		return err;
	}
	return 0;
}

static void hello_exit(void)
{
	if (khello_task != NULL) {
		kthread_stop(khello_task);
	}
	khello_task = NULL;
}

module_init(hello_init);
module_exit(hello_exit);
